import React, { Component } from 'react'
import PropTypes from 'prop-types'
import isEqual from 'lodash/isEqual'
import pick from 'lodash/pick'
import noop from 'lodash/noop'

import fbInit from './../../utils/fbInit'

import * as MessengerPluginEvents from './../../constants/MessengerPluginEvents'
import * as ClassNames from './../../constants/ClassNames'


export default class CustomerChatPlugin extends Component {
  shouldComponentUpdate(props) {
    this.shouldPluginUpdate(props)

    const componentAffectProps = [
      'className', 'pluginClassName',
      'pageId', 'dataRef', 'mimimize',
    ]
    return !isEqual(pick(this.props, componentAffectProps), pick(props, componentAffectProps))
  }

  shouldPluginUpdate(props) {
    const pluginAffectProps = [
      'pageId', 'dataRef', 'mimimize',
    ]

    const isPluginAffected = !isEqual(pick(this.props, pluginAffectProps), pick(props, pluginAffectProps))

    if (isPluginAffected) {
      fbInit(props)
    }
  }

  componentDidMount() {
    fbInit(this.props)
    this.subscribe()
  }

  componentWillUnmount() {
    this.unsubscribe()
  }

  subscribe() {
    window.FB.Event.subscribe('customerchat', this.handleEvents.bind(this))
  }

  unsubscribe() {
    window.FB.Event.unsubscribe('customerchat', this.handleEvents.bind(this))
  }

  handleEvents(event) {
    if (!event.ref || event.ref !== this.props.dataRef) {
      return
    }

    if (event.event === MessengerPluginEvents.HIDDEN) {
      this.props.onHide(event)
    }

    if (event.event === MessengerPluginEvents.RENDERED) {
      this.props.onRender(event)
    }
  }

  getSnippet() {
    const { id, pageId, dataRef, mimimize } = this.props

    return (
      `<div
          id='${id}'
          class='${ClassNames.CUSTOMER_CHAT_PLUGIN}'
          page_id='${pageId}'
          data-ref='${dataRef}'
          ${mimimize ? `mimimize='${mimimize}'` : ''}></div>`
    )
  }

  render() {
    const { className, pluginClassName } = this.props

    return (
      <div className={className}>
        <div className={pluginClassName} dangerouslySetInnerHTML={{ __html: this.getSnippet() }}></div>
      </div>
    )
  }
}


CustomerChatPlugin.propTypes = {
  className: PropTypes.string,
  pluginClassName: PropTypes.string,

  id: PropTypes.string,
  pageId: PropTypes.string.isRequired,
  dataRef: PropTypes.string.isRequired,
  mimimize: PropTypes.bool,

  onHide: PropTypes.func,
  onRender: PropTypes.func,
}


CustomerChatPlugin.defaultProps = {
  id: '',
  className: '',
  pluginClassName: '',

  mimimize: null,

  onHide: noop,
  onRender: noop,
}