import React, { Component } from 'react'
import PropTypes from 'prop-types'
import values from 'lodash/values'
import isEqual from 'lodash/isEqual'
import pick from 'lodash/pick'

import fbInit from './../../utils/fbInit'

import * as ShareButtonSize from './../../constants/ShareButtonSize'
import * as ShareButtonLayout from './../../constants/ShareButtonLayout'
import * as ClassNames from './../../constants/ClassNames'


export default class ShareButton extends Component {
  shouldComponentUpdate(props) {
    this.shouldPluginUpdate(props)

    const componentAffectProps = ['href', 'layout', 'mobileIframe', 'size', 'className']
    return !isEqual(pick(this.props, componentAffectProps), pick(props, componentAffectProps))
  }

  shouldPluginUpdate(props) {
    const pluginAffectProps = ['href', 'layout', 'mobileIframe', 'size']

    const isPluginAffected = !isEqual(pick(this.props, pluginAffectProps), pick(props, pluginAffectProps))

    if (isPluginAffected) {
      fbInit(props)
    }
  }

  componentDidMount() {
    fbInit(this.props)
  }

  getSnippet() {
    const { href, layout, mobileIframe, size } = this.props

    return (
      `<div
        class='${ClassNames.SHARE_BUTTON}'
        data-href='${href}'
        data-layout='${layout}'
        data-mobile_iframe='${JSON.stringify(mobileIframe)}'
        data-size='${size}'></div>`
    )
  }

  render() {
    return (
      <div className={this.props.className}>
        <div dangerouslySetInnerHTML={{ __html: this.getSnippet() }}></div>
      </div>
    )
  }
}


ShareButton.propTypes = {
  appId: PropTypes.string.isRequired,
  href: PropTypes.string.isRequired,
  layout: PropTypes.oneOf(values(ShareButtonLayout)),
  mobileIframe: PropTypes.bool.isRequired,
  size: PropTypes.oneOf(values(ShareButtonSize)),
  className: PropTypes.string,
}


ShareButton.defaultProps = {
  mobileIframe: false,
  layout: ShareButtonLayout.BUTTON,
  size: ShareButtonSize.SMALL,
}
